#!/usr/bin/env python

import sys,os,getopt,stat
import xml.sax
import re
import operator

# -------------------------------------------------------------------------------------------------
# *** FUNCTION info <<<
# -------------------------------------------------------------------------------------------------
def info(s,level=1):
    """ @brief Print info to stdout
        @param s string to print
    """
    if VERBOSE>=level:
        sys.stderr.write("*II* %s\n" % s)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION info >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION warning <<<
# -------------------------------------------------------------------------------------------------
def warning(s):
    """ @brief Print warning to stdout
        @param s string to print
    """
    if WARNING:
        sys.stderr.write("*WW* %s\n" % s)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION warning >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION error <<<
# -------------------------------------------------------------------------------------------------
def error(s):
    """ @brief Print error to stdout
        @param s string to print
    """
    print "*EE* %s " % s
    sys.exit(1)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION error >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION usage <<<
# -------------------------------------------------------------------------------------------------
def usage(myname):
    print "\nUsage:   %s OPTIONS <XML_FILE>\n" % (myname)

    print "Read configuration from an XML file and create config files."

    print """
%s reads a data base of configuration parameters from an XML file.
Subsequently, a number of template files are processed in order to create
configuration files (targets). See ec-conf User Guide for more information.""" % (myname)

    print """
Options: -h|--help            Print this help screen.
         -p|--platform        Set the active platform.
         -g|--gui             Starts the graphical user interface. Turns off -x and -l.
         -x|--write-xml       Writes the content of <XML_FILE> in XML format to stdout.
                              This can be used to normalise the XML file and for test purposes.
         -l|--list-platforms  Lists all platforms, which are defined in the XML file, on stdout.
         -v|--verbose         Produces verbose output (stderr). To increase verbosity,
                                use more than once.
         -w|--no-warning      Turns off warnings (however, errors are displayed).
          """
    sys.exit(2)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION usage >>>
# -------------------------------------------------------------------------------------------------

# =================================================================================================
# *** CLASS TextNode <<<
# =================================================================================================
class TextNode(object):
    def __init__(self):
        self.text = ""

    def addText(self,content):
        self.text += content
# =================================================================================================
# *** END of CLASS TextNode >>>
# =================================================================================================

# =================================================================================================
# *** CLASS NamedNode <<<
# =================================================================================================
class NamedNode(object):

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __init__ <<<
# -------------------------------------------------------------------------------------------------
    def __init__(self,attr=None):

        self.name = None
        self.description = None

        if attr:
            if 'name' in attr.keys():
                self.name = attr['name']

        info('Create named node: '+self.name+' ('+self.__class__.__name__+')',level=2)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __init__ >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION set <<<
# -------------------------------------------------------------------------------------------------
    def set(self,key,value):
        self.__dict__.__setitem__(key,value)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION set >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION xml <<<
# -------------------------------------------------------------------------------------------------
    def xml(self,level=0):
        """Provides XML representation of the NamedNode as a string"""

        tabwidth = 4

        s = ' '*level*tabwidth
        s += '<' + self.__class__.__name__
        if 'name' in self.__dict__ and self.name:
            s += ' name="' + self.name + '"'
        s += '>\n'

        for e in 'description','template','target','properties','type','value':
            if e in self.__dict__:
                s += ' '*(level+1)*tabwidth
                s += '<' + e.capitalize() + '>'
                if self.__dict__[e]: s += self.__dict__[e]
                s += '</' + e.capitalize() + '>\n'

        if level<=1: s+='\n'

        for e in 'translation','platform','model','parameter':
            if e in self.__dict__:
                for t in self.__dict__[e]:
                    s += t.xml(level+1)

        s += ' '*level*tabwidth
        s += '</' + self.__class__.__name__ + '>\n'

        if level>0 and level<=2: s += '\n'

        return s
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION xml >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION plt <<<
# -------------------------------------------------------------------------------------------------
    def plt(self,name=None):
        """Provides short-hand access to the Platform member with name 'name'."""
        if 'platform' in self.__dict__:
            if name:
                for p in self.platform:
                    if p.name == name:
                        return p
            else:
                return self.platform
        return None
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION plt >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION mod <<<
# -------------------------------------------------------------------------------------------------
    def mod(self,name=None):
        """Provides short-hand access to the Model member with name 'name'."""

        if 'model' in self.__dict__:
            if name:
                for m in self.model:
                    if m.name == name:
                        return m
            else:
                return self.model
        return None
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION mod >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION par <<<
# -------------------------------------------------------------------------------------------------
    def par(self,name=None):
        """Provides short-hand access to the Parameter member with name 'name'."""

        if 'parameter' in self.__dict__:
            if name:
                for p in self.parameter:
                    if p.name == name:
                        return p
            else:
                return self.parameter
        return None
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION par >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION tra <<<
# -------------------------------------------------------------------------------------------------
    def tra(self,name=None):
        """Provides short-hand access to the Translation member with name 'name'."""

        if 'translation' in self.__dict__:
            if name:
                for t in self.translation:
                    if t.name == name:
                        return t
            else:
                return self.translation
        return None
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION mod >>>
# -------------------------------------------------------------------------------------------------


# =================================================================================================
# *** END of CLASS NamedNode >>>
# =================================================================================================

# =================================================================================================
# *** CLASS Translation <<<
# =================================================================================================
class Translation(NamedNode):
    def __init__(self,attr):
        super(Translation,self).__init__(attr)
        self.template   = None
        self.target     = None
        self.properties = ''
        self.isActive   = 1

    def setTarget(self,t):
        self.target = t
# =================================================================================================
# *** END of CLASS Translation >>>
# =================================================================================================

# =================================================================================================
# *** CLASS Platform <<<
# =================================================================================================
class Platform(NamedNode):
    def __init__(self,attr):
        super(Platform,self).__init__(attr)
        self.parameter   = []
        self.translation = []
# =================================================================================================
# *** END of CLASS Platform >>>
# =================================================================================================

# =================================================================================================
# *** CLASS Model <<<
# =================================================================================================
class Model(NamedNode):
    def __init__(self,attr):
        super(Model,self).__init__(attr)
        self.parameter = []
# =================================================================================================
# *** END of CLASS Model >>>
# =================================================================================================

# =================================================================================================
# *** CLASS Parameter <<<
# =================================================================================================
class Parameter(NamedNode):
    def __init__(self,attr):
        super(Parameter,self).__init__(attr)
        self.type  = None
        self.value = None
# =================================================================================================
# *** END of CLASS Parameter >>>
# =================================================================================================

# =================================================================================================
# *** CLASS Configuration <<<
# =================================================================================================
class Configuration(NamedNode,xml.sax.handler.ContentHandler):

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __init__ <<<
# -------------------------------------------------------------------------------------------------
    def __init__(self):

        self.translation = []
        self.platform    = []
        self.model       = []

        self.xmlFile        = None
        self.activePlatform = None

        self.__stack = []
        self.__types = { 'Translation' : Translation,
                         'Platform'    : Platform,
                         'Model'       : Model,
                         'Parameter'   : Parameter }
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __init__ >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION xml <<<
# -------------------------------------------------------------------------------------------------
    def xml(self,level=0):

        s = '<?xml version="1.0"?>\n\n'
        s += super(Configuration,self).xml(level)

        return s
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION xml >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION save <<<
# -------------------------------------------------------------------------------------------------
    def save(self):

        #if os.path.isfile(self.xmlFile):
            #msg = "The file '"+self.xmlFile+"' exists. Do you want to replace it?"
            #if not tkMessageBox.askyesno('Save XML file',msg): return

        try: f = open(self.xmlFile,'w')
        except IOError:
            msg = "The file '"+self.xmlFile+"' could not be opened for writing"
            error('Save XML file',msg)
            info("XML database NOT saved")
            return

        f.write(self.xml())
        f.close()
        info("XML database saved to file '"+self.xmlFile+"'")

        return True

# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION save >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION parseVarSet <<<
# -------------------------------------------------------------------------------------------------

    def parseVarSet(self,s,v): # <<<

        varRE   = re.compile(r"^(\w{3}):([a-zA-Z0-9_]+):([a-zA-Z0-9_]+)$")

        while 1:

            m = varRE.search(s)

            if not m: break

            (category,component,parameter) = m.groups()

            if category.lower()=='plt' and component.lower() == 'active':
                component = self.activePlatform

            try:
                if category.lower()=='tra':
					t = getattr(self,category.lower())(parameter)
					s = getattr(self,category.lower())(parameter).target
					t.setTarget(v)
                else:
					s = getattr(self,category.lower())(component).par(parameter).value
					p = getattr(self,category.lower())(component).par(parameter)
					p.set('value',v)
            except:
                warning("Unable to process '%s'" % s)
                return '[[['+s+']]]'
        return s
        # >>>
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION parseVarSet >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION startElement <<<
# -------------------------------------------------------------------------------------------------
    def startElement(self,tag,attributes):

        info("Processing XML element '"+str(tag)+"'",level=3)

        if self.__stack:
            if tag in self.__types:
                info("Adding NamedNode for element '"+str(tag)+"'",level=3)
                self.__stack.append(self.__types[tag](attributes))
            else:
                info("Adding TextNode for element '"+str(tag)+"'",level=3)
                self.__stack.append(TextNode())
        else:
            self.__stack.append(self)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION startElement >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION characters <<<
# -------------------------------------------------------------------------------------------------
    def characters(self,content):
        if isinstance(self.__stack[-1],TextNode):
            self.__stack[-1].addText(content)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION characters >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION endElement <<<
# -------------------------------------------------------------------------------------------------
    def endElement(self,tag):
        n = self.__stack.pop()
        if self.__stack:
            if isinstance(n,TextNode):
                self.__stack[-1].__dict__[tag.lower()] = n.text
            else:
                self.__stack[-1].__dict__[tag.lower()].append(n)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION endElement >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION parse <<<
# -------------------------------------------------------------------------------------------------
    def parse(self,file):

        parser = xml.sax.make_parser()
        parser.setContentHandler(self)

        info("Parsing XML file '%s'" % (file))

        try: parser.parse(file)
        except IOError:
            error("Could not open file '"+file+"' for reading")

        info("Finished parsing '%s': Translation: %d Platform: %d Model: %d" %
                                   (file,len(self.translation),len(self.platform),len(self.model)))
        self.xmlFile = file
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION parse >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION translate <<<
# -------------------------------------------------------------------------------------------------
    def translate(self,translation):

        RPN_OPS = {
            'ADD' : operator.add,
            'SUB' : operator.sub,
            'MUL' : operator.mul,
            'DIV' : operator.truediv,
            'POW' : operator.pow,
            'MOD' : operator.mod
        }

        substRE = re.compile(r"\[\[\[(?P<var>[a-zA-Z0-9_:,]+)\]\]\]")
        varRE   = re.compile(r"^(\w{3}):([a-zA-Z0-9_]+):([a-zA-Z0-9_]+)$")

        def parseVar(s): # <<<

            while 1:

                m = varRE.search(s)

                if not m: break

                (category,component,parameter) = m.groups()

                if category.lower()=='plt' and component.lower() == 'active':
                    component = self.activePlatform

                try: s = getattr(self,category.lower())(component).par(parameter).value
                except:
                    warning("Unable to process '%s' (Line %d in '%s')" % (s,line_number,template))
                    return '[[['+s+']]]'
            return s
            # >>>

        def parseRNP(s): # <<<

            stack = []

            for token in s.split(','):

                token = parseVar(token)
                if not token:
                    warning("Substitute expression with empty string (Line %d in '%s')" % (line_number,template))

                try: result = int(token)
                except ValueError:

                    try: result = float(token)
                    except ValueError:

                        if token in RPN_OPS.keys():

                            try: result = RPN_OPS[token](stack.pop(-2),stack.pop())
                            except IndexError:
                                warning("Too few arguments to execute '%s' (Line %d in '%s')" % (token,line_number,template))
                                return "[[["+s+"]]]"
                            except:
                                warning("Unable to execute '%s' (Line %d in '%s')" % (token,line_number,template))
                                return "[[["+s+"]]]"
                        else:
                            result = token

                stack.append(result)

            if len(stack)>1:
                warning("Too many operands in '%s' (Line %d in '%s')" % (s,line_number,template))
                return "[[["+s+"]]]"

            return result
            # >>>

        template = translation.template
        target   = translation.target.strip()

        info("Translate: '%s' --> '%s'" % (template,target))

        try: input = open(template,'r')
        except IOError:
            error("Can't open template file '%s' for reading" % (template))

        if target:
            try: output = open(target,'w')
            except IOError:
                error("Can't open target file '%s' for writing" % (target))

            if 'executable' in translation.properties.split(','):
                os.chmod(target,stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH | \
                                stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH | \
                                stat.S_IWUSR)
        else:
            output = sys.stdout

        line_number = 0
        for line in input:
            line_number += 1
            ptr = 0
            buf = ''
            for m in substRE.finditer(line):
                buf += line[ptr:m.start()]
                var  = m.groupdict()['var']
                buf += str(parseRNP(var))
                ptr  = m.end()
            buf += line[ptr:]

            output.write(buf)

        input.close()
        if target: output.close()

# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION translate >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION translateAll <<<
# -------------------------------------------------------------------------------------------------
    def translateAll(self):
        for t in self.translation:
            self.translate(t)

        if self.activePlatform:
            for t in self.plt(self.activePlatform).translation:
                self.translate(t)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION translateAll >>>
# -------------------------------------------------------------------------------------------------

# =================================================================================================
# *** END of CLASS Configuration >>>
# =================================================================================================

# -------------------------------------------------------------------------------------------------
# *** FUNCTION startGUI <<<
# -------------------------------------------------------------------------------------------------
def startGUI(cfg):

    import Tkinter
    import tkFileDialog
    import tkMessageBox

# =================================================================================================
# *** CLASS VerticalScrolledFrame <<<
# =================================================================================================
    class VerticalScrolledFrame(Tkinter.Frame):
        """A pure Tkinter scrollable frame that actually works!

        * Use the 'interior' attribute to place widgets inside the scrollable frame
        * Construct and pack/place/grid normally
        * This frame only allows vertical scrolling

        http://tkinter.unpythonic.net/wiki/VerticalScrolledFrame

        """
        def __init__(self, parent, *args, **kw):
            Tkinter.Frame.__init__(self, parent, *args, **kw)

            # create a canvas object and a vertical scrollbar for scrolling it
            vscrollbar = Tkinter.Scrollbar(self, orient=Tkinter.VERTICAL)
            vscrollbar.pack(fill=Tkinter.Y, side=Tkinter.RIGHT, expand=Tkinter.FALSE)
            canvas = Tkinter.Canvas(self, bd=0, highlightthickness=0,
                                          yscrollcommand=vscrollbar.set)
            canvas.pack(side=Tkinter.LEFT, fill=Tkinter.BOTH, expand=Tkinter.TRUE)
            vscrollbar.config(command=canvas.yview)

            # reset the view
            canvas.xview_moveto(0)
            canvas.yview_moveto(0)

            # create a frame inside the canvas which will be scrolled with it
            self.interior = interior = Tkinter.Frame(canvas)
            interior_id = canvas.create_window(0, 0, window=interior,
                                               anchor=Tkinter.NW)

            # track changes to the canvas and frame width and sync them,
            # also updating the scrollbar
            def _configure_interior(event):
                # update the scrollbars to match the size of the inner frame
                size = (interior.winfo_reqwidth(), interior.winfo_reqheight())
                canvas.config(scrollregion="0 0 %s %s" % size)
                if interior.winfo_reqwidth() != canvas.winfo_width():
                    # update the canvas's width to fit the inner frame
                    canvas.config(width=interior.winfo_reqwidth())
            interior.bind('<Configure>', _configure_interior)

            def _configure_canvas(event):
                if interior.winfo_reqwidth() != canvas.winfo_width():
                    # update the inner frame's width to fill the canvas
                    canvas.itemconfigure(interior_id, width=canvas.winfo_width())
            canvas.bind('<Configure>', _configure_canvas)

            return
# =================================================================================================
# *** END of CLASS VerticalScrolledFrame >>>
# =================================================================================================

# =================================================================================================
# *** CLASS GUI <<<
# =================================================================================================
    class GUI(Tkinter.Tk):

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __init__ <<<
# -------------------------------------------------------------------------------------------------
        def __init__(self,cfg):

            Tkinter.Tk.__init__(self)

            self.columnconfigure(1,weight=1)
            self.rowconfigure(0,pad=10)
            self.rowconfigure(2,weight=1)

            self.__cfg      = cfg
            self.__varList  = []
            self.__varDict  = {}

            for t in self.__cfg.translation + [t for p in self.__cfg.platform for t in p.translation]:
                v = Tkinter.IntVar()
                v.set(1)
                v.trace('w',lambda n,i,m,t=t: t.set('isActive',int(self.globalgetvar(n))))
                self.__addVar(v,t)

            self.__activeComponent = Tkinter.StringVar()

            self.__status = Tkinter.StringVar()

            self.__componentFrame   = None
            self.__parameterFrame   = None
            self.__translationFrame = None

            self.__initTopPanel()
            self.__initStatusPanel()
            self.__initMainPanel()

            self.grid_propagate(flag=0)

            self.__setStatus("Welcome to EC-CONF's graphical user interface!",time=3000)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __init__ >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __addVar <<<
# -------------------------------------------------------------------------------------------------
        def __addVar(self,var,obj=None):
            self.__varList.append(var)
            if obj:
                self.__varDict[obj] = len(self.__varList)-1
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __addVar >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __setStatus <<<
# -------------------------------------------------------------------------------------------------
        def __setStatus(self,message,time=8000):

            try:    self.after_cancel(self.__statusAfterID)
            except: pass

            self.__defaultStatusMessage = 'Basic usage: SELECT XML database and template/target files, ' \
                                                      ' CONFIGURE parameters, ' \
                                                      ' and CREATE the configuration files.'
            self.__status.set(message)
            self.__statusAfterID = self.after(time,self.__status.set,self.__defaultStatusMessage)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __setStatus >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __initTopPanel <<<
# -------------------------------------------------------------------------------------------------
        def __initTopPanel(self):

            w = Tkinter.Label(text='The active platform is:')
            w.grid(row=0,column=0)

            if not self.__cfg.activePlatform:
                self.__cfg.activePlatform = self.__cfg.platform[0].name

            v = Tkinter.StringVar()
            v.set(self.__cfg.activePlatform)
            v.trace('w',lambda n,i,m: self.__setStatus("Active platform changed to '"+self.__cfg.activePlatform+"'"))
            v.trace('w',lambda n,i,m: self.__fillParameterFrame(self.__parameterFrame))
            v.trace('w',lambda n,i,m: self.__fillComponentFrame(self.__componentFrame))
            v.trace('w',lambda n,i,m: self.__fillTranslationFrame(self.__translationFrame))
            v.trace('w',lambda n,i,m: self.__cfg.set('activePlatform',self.globalgetvar(n)))
            self.__addVar(v)

            w = Tkinter.OptionMenu(None,v,*[p.name for p in self.__cfg.platform])
            w.grid(row=0,column=1,sticky='W',padx=8)

            w = Tkinter.Button(text='Select',width=10,height=2,bg='tan')
            w['command'] = self.__doSelect
            w.grid(row=0,column=2,padx=8)

            w = Tkinter.Button(text='Configure',width=10,height=2,bg='tan')
            w['command'] = self.__doConfigure
            w.grid(row=0,column=3,padx=8)

            w = Tkinter.Button(text='Create!',width=10,height=2,bg='darkgrey',fg='white')
            w['command'] = self.__doCreate
            w.grid(row=0,column=4,padx=8)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __initTopPanel >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION  __initStatusPanel<<<
# -------------------------------------------------------------------------------------------------
        def __initStatusPanel(self):
            w = Tkinter.Label(textvariable=self.__status,height=2,bg='orange')
            w.grid(row=1,column=0,columnspan=5,sticky='EW',pady=5)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __initStatusPanel >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __initMainPanel <<<
# -------------------------------------------------------------------------------------------------
        def __initMainPanel(self):

            self.__initSelectPanel()
            self.__initConfigurePanel()

            self.__selectPanel.grid_remove()
            self.__configurePanel.grid_remove()

            self.__activeMainPanel = self.__selectPanel
            self.__activeMainPanel.grid()
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __initMainPanel >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __initSelectPanel <<<
# -------------------------------------------------------------------------------------------------
        def __initSelectPanel(self):

            self.__selectPanel = Tkinter.Frame()
            self.__selectPanel.grid(row=2,column=0,columnspan=5,sticky='NEWS')
            self.__selectPanel.columnconfigure(0,weight=1)
            self.__selectPanel.rowconfigure(2,weight=1)

            f = Tkinter.LabelFrame(self.__selectPanel,text='XML database file')
            f.grid(sticky='NEWS')
            f.columnconfigure(1,weight=1)

            w = Tkinter.Label(f,text='The current file is: ')
            w.grid(row=0,column=0,sticky='W')

            v = Tkinter.StringVar()
            v.set(self.__cfg.xmlFile)
            v.trace('w',lambda n,i,m: self.__cfg.set('xmlFile',self.globalgetvar(n)))
            self.__addVar(v,'xmlFile')

            w = Tkinter.Label(f,textvariable=v,bg='darkgrey',fg='white')
            w.grid(row=0,column=1,sticky='W')

            w = Tkinter.Button(f,text='Save as',width=8)
            w['command'] = lambda :self.__saveAsXmlFile()
            w.grid(row=0,column=2,sticky='E',padx=4,pady=5)

            w = Tkinter.Button(f,text='Save',width=8)
            w['command'] = lambda :self.__saveXmlFile()
            w.grid(row=0,column=3,sticky='E',padx=4,pady=5)

            Tkinter.Frame(self.__selectPanel).grid(pady=5)

            self.__translationFrame = Tkinter.LabelFrame(self.__selectPanel,text='Templates and Targets')
            self.__translationFrame.grid(sticky='NEWS')
            self.__translationFrame.columnconfigure(1,weight=1)
            self.__translationFrame.columnconfigure(3,weight=9)

            self.__fillTranslationFrame(self.__translationFrame)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __initSelectPanel >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __initConfigurePanel <<<
# -------------------------------------------------------------------------------------------------
        def __initConfigurePanel(self):

            self.__configurePanel = Tkinter.Frame()
            self.__configurePanel.grid(row=2,column=0,columnspan=5,sticky='NEWS')
            self.__configurePanel.rowconfigure(0,weight=1)
            self.__configurePanel.columnconfigure(2,weight=1)

            # Component frame
            self.__componentFrame = Tkinter.LabelFrame(self.__configurePanel,text='Configurable components')
            self.__componentFrame.grid(sticky='NEWS')

            self.__fillComponentFrame(self.__componentFrame)

            # Spacer frame
            f = Tkinter.Frame(self.__configurePanel)
            f.grid(row=0,column=1,padx=2)

            # Parameter frame
            f = Tkinter.LabelFrame(self.__configurePanel,text='Configuration parameters')
            f.grid(row=0,column=2,sticky='NEWS')

            f = VerticalScrolledFrame(f)
            f.pack(fill=Tkinter.BOTH,expand=Tkinter.TRUE)

            self.__parameterFrame = f.interior
            self.__parameterFrame.columnconfigure(2,weight=1)

            self.__fillParameterFrame(self.__parameterFrame)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __initConfigurePanel >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __fillTranslationFrame <<<
# -------------------------------------------------------------------------------------------------
        def __fillTranslationFrame(self,frame):

            for w in frame.children.values():
                w.destroy()

            v = Tkinter.IntVar()
            v.set(1)
            v.trace('w',lambda n,i,m: [ self.__varList[self.__varDict[t]].set(self.globalgetvar(n))
                                            for t in self.__cfg.translation +
                                                     self.__cfg.plt(self.__cfg.activePlatform).translation ])
            self.__addVar(v)

            w = Tkinter.Checkbutton(frame,text='Activate/deactivate all',variable=v)
            w.grid(row=0,column=0,sticky='W',pady=5)

            r = 1
            for t in self.__cfg.translation + self.__cfg.plt(self.__cfg.activePlatform).translation:

                w = Tkinter.Checkbutton(frame,text=t.description,variable=self.__varList[self.__varDict[t]])
                w.grid(row=r,column=0,sticky='W',pady=5)

                w = Tkinter.Label(frame,text=t.template,bg='darkgrey',fg='white')
                w.grid(row=r,column=1,sticky='E')

                w = Tkinter.Label(frame,text=' --> ')
                w.grid(row=r,column=2)

                v = Tkinter.StringVar()
                v.set(t.target)
                v.trace('w',lambda n,i,m,t=t: t.setTarget(self.globalgetvar(n)))
                self.__addVar(v)
                w = Tkinter.Entry(frame,textvariable=v)

                w.grid(row=r,column=3,sticky='EW')
                r+=1

# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __fillTranslationFrame >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __fillComponentFrame <<<
# -------------------------------------------------------------------------------------------------
        def __fillComponentFrame(self,frame):

            for w in frame.children.values():
                w.destroy()

            if self.__cfg.activePlatform:

                self.__activeComponent.set(self.__cfg.activePlatform)

                w = Tkinter.Label(frame,text='Active platform')
                w.pack(anchor='w')

                c = self.__cfg.plt(self.__cfg.activePlatform)
                w = Tkinter.Radiobutton(frame,text=c.name,variable=self.__activeComponent,value=c.name)
                w['command'] = lambda:self.__fillParameterFrame(self.__parameterFrame)
                w.pack(anchor='w',pady=5)

            if self.__cfg.model:

                if not self.__activeComponent.get():
                    self.set(self.__cfg.model[0].name)

                w = Tkinter.Label(frame,text='Configurable models')
                w.pack(anchor='w')

                for c in self.__cfg.model:
                    w = Tkinter.Radiobutton(frame,text=c.name,variable=self.__activeComponent,value=c.name)
                    w['command'] = lambda:self.__fillParameterFrame(self.__parameterFrame)
                    w.pack(anchor='w',pady=5)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __fillComponentFrame >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __fillParameterFrame <<<
# -------------------------------------------------------------------------------------------------
        def __fillParameterFrame(self,frame):

            for w in frame.children.values():
                w.destroy()

            for (name,component) in [(c.name,c) for c in self.__cfg.platform + self.__cfg.model]:
                if self.__activeComponent.get()==name:
                    break

            r = 0
            for p in component.par():

                w = Tkinter.Label(frame,text=p.description,anchor="w",width=35)
                w.grid(row=r,column=0,sticky='W')

                w = Tkinter.Label(frame,text='['+str(p.name)+']')
                w.grid(row=r,column=1,sticky='W',padx=20)

                v = Tkinter.StringVar()
                v.set(p.value)
                v.trace('w',lambda n,i,m,p=p: p.set('value',self.globalgetvar(n)))
                self.__addVar(v)
                w = Tkinter.Entry(frame,textvariable=v)
                w.grid(row=r,column=2,sticky='EW')

                r+=1

            self.__setStatus('Configure parameters for component \''+self.__activeComponent.get()+'\'')
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __fillParameterFrame >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __doSelect <<<
# -------------------------------------------------------------------------------------------------
        def __doSelect(self):
            self.__activeMainPanel.grid_remove()
            self.__activeMainPanel = self.__selectPanel
            self.__activeMainPanel.grid()
            self.__setStatus('Select the XML data base file and active translations in the panel below.')
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __doSelect >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __doConfigure <<<
# -------------------------------------------------------------------------------------------------
        def __doConfigure(self):
            self.__activeMainPanel.grid_remove()
            self.__activeMainPanel = self.__configurePanel
            self.__activeMainPanel.grid()
            self.__setStatus('Configure the configuration parameters for the available components in the panel below.')
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __doConfigure >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __doCreate <<<
# -------------------------------------------------------------------------------------------------
        def __doCreate(self):

            fw = []
            for t in self.__cfg.translation + self.__cfg.plt(self.__cfg.activePlatform).translation:
                if t.isActive:
                    self.__cfg.translate(t)
                    fw.append(t.target)

            if fw:
                msg = 'Active target files written: '+fw.pop()
                while fw: msg += ', '+fw.pop()
            else:
                msg = 'No targets where written'

            self.__setStatus(msg)
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __doCreate >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __saveAsXmlFile <<<
# -------------------------------------------------------------------------------------------------
        def __saveAsXmlFile(self):
            f = tkFileDialog.asksaveasfilename(title='Select a file name for saving:',
                                               filetypes=[('XML files','*.xml'),('All files','*')])
            if f:
                try:
                    self.__varList[self.__varDict['xmlFile']].set(os.path.relpath(f))
                except AttributeError:
                    self.__varList[self.__varDict['xmlFile']].set(os.path.realpath(f))
                self.__saveXmlFile()
            else:
                self.__setStatus("Current XML file NOT saved")

# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __saveAsXmlFile >>>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# *** FUNCTION __saveXmlFile <<<
# -------------------------------------------------------------------------------------------------
        def __saveXmlFile(self):

            if os.path.isfile(self.__cfg.xmlFile):
                msg = "The file '"+self.__cfg.xmlFile+"' exists. Do you want to replace it?"
                if not tkMessageBox.askyesno('Save XML file',msg): return

            try: f = open(self.__cfg.xmlFile,'w')
            except IOError:
                msg = "The file '"+self.__cfg.xmlFile+"' could not be opened for writing"
                tkMessageBox.showerror('Save XML file',msg)
                self.__setStatus("XML database NOT saved")
                return

            f.write(self.__cfg.xml())
            f.close()
            self.__setStatus("XML database saved to file '"+self.__cfg.xmlFile+"'")
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION __saveXmlFile >>>
# -------------------------------------------------------------------------------------------------

# =================================================================================================
# *** END of CLASS GUI >>>
# =================================================================================================

    root = GUI(cfg)
    root.title('ec-conf GUI')

    minWindowWidth  = min(900,int(0.9*root.winfo_screenwidth()))
    minWindowHeight = min(800,int(0.9*root.winfo_screenheight()))
    root.minsize(minWindowWidth,minWindowHeight)

    root.resizable()
    root.mainloop()
# -------------------------------------------------------------------------------------------------
# *** END of FUNCTION startGUI >>>
# -------------------------------------------------------------------------------------------------

if __name__ == "__main__":

    # Try to get command line options and arguments
    try: opts,args = getopt.getopt(sys.argv[1:],"hp:gxstflvw",["help","platform=","gui","write-xml",
                                                             "set-parameter","parameter=","value=",
                                                             "list-platforms","verbose","no-warning"])
    except getopt.GetoptError:
        usage(os.path.split(sys.argv[0])[-1])

    # Default values, to be overwritten by command line options
    WARNING       = 1
    VERBOSE       = 0
    writeXML      = False
    setParameter  = False
    parameter     = None
    value         = None
    listPlatforms = False
    wantGUI       = False
    platform      = None

    # Parse command line options
    for opt,arg in opts:
        if   opt in ('-h','--help'):
            usage(os.path.split(sys.argv[0])[-1])
        elif opt in ('-p','--platform'):
            platform = arg
        elif opt in ('-g','--gui'):
            wantGUI = True
        elif opt in ('-x','--write-xml'):
            writeXML = True
        elif opt in ('-s','--set-parameter'):
            setParameter = True
        elif opt in ('-t','--parameter'):
            parameter = arg
        elif opt in ('-f','--value'):
            value = arg
        elif opt in ('-l','--list-platforms'):
            listPlatforms = True
        elif opt in ('-v','--verbose'):
            VERBOSE += 1
        elif opt in ('-w','--no-warning'):
            WARNING=0

    # The XML file is all that should be left on the command line
    if len(args)!=1:
        usage(os.path.split(sys.argv[0])[-1])

    # Create the Configuration object and fill the data structures by parsing the XML file
    cfg = Configuration()

    try: cfg.parse(args[0])
    except xml.sax._exceptions.SAXParseException, e:
        msg  = 'XML parse error: '
        msg += "'"+e._msg+"'"
        msg += " (line %s, column %s in file '%s')" % (e._linenum,e._colnum,e._systemId)
        error(msg)

    # If a platform was given on the command line, try to set it
    if platform:
        if cfg.plt(platform):
            cfg.activePlatform = platform
        else:
            error("Platform '%s' not defined in the configuration file '%s'" % (platform,args[0]))
    elif not (wantGUI or listPlatforms):
        warning("No active platform given")

    # Select activity to be done according to the command line options
    # Default is to translate all Translations in the Configuration
    if wantGUI:
        info("Starting GUI")
        startGUI(cfg)
    elif listPlatforms:
        if cfg.plt():
            print '\n'.join([p.name for p in cfg.plt()])
        else:
            warning("No platforms defined in XML file")
    elif writeXML:
        sys.stdout.write(cfg.xml())
    elif setParameter:
        cfg.parseVarSet(parameter,value)
        if not cfg.save():
            sys.stdout.write(cfg.xml())
    else:
        cfg.translateAll()

