#!/usr/bin/env python
import argparse
from pkg_resources import resource_exists
import shutil
import subprocess
import os
import textwrap
from autosubmit.config.basicConfig import BasicConfig
from autosubmit.autosubmit import Autosubmit
from autosubmit.config.config_common import AutosubmitConfig

RESULT = '\033[32m'
WARNING = '\033[33m'
ERROR = '\033[31m'
CRITICAL = '\033[1m \033[31m'
DEFAULT = '\033[0m\033[39m'


def main():
    """
    OBJECTIVE:
    In Autosubmit 2.x experiments, three projects were cloned: model, templates and ocean_diagnostics.
    New Autosubmit 3.x experiments, only allow cloning one project: auto-ecearth.
    Auto-ecearth is a single project containing model, templates and ocean_diagnostics among others.

    This script covers the need of having to run an Autosubmit 2.x experiment with Autosubmit 3.x.
    It assumes that no auto-ecearth available version can reproduce what the old experiment was doing,
    and it adapts a copy of that old experiment, reshuffling the configuration files and removing
    unnecessary variables from the old templates to adapt them to the Autosubmit 3.x framework.

    If overwrite option is provided, the modifications are performed in the original experiment.

    DESCRIPTION:
    A copy of the ORIGINAL_EXPID is performed in a new TARGET_EXPID. If overwrite option is provided
    the ORIGINAL_EXPID will be assumed to be the TARGET_EXPID.
    WARNING: Use overwrite option carefully!

    This script will modify and adapt the experiment configuration file of the TARGET_EXPID
    according to Autosubmit 3.x standard.

    This script will also clone model, templates and ocean_diagnostics
    from ORIGINAL_EXPID into TARGET_EXPID "proj" folder.

    This script will also remove unnecessary template header/tailer variables.

    USE:
    python migrate_exp.py -f ORIGINAL_EXPID [-o]
    """

    parser = argparse.ArgumentParser(description='migration script for experiments from '
                                                 'Autosubmit 2.x to Autosubmit 3.x. ')
    parser.add_argument('-f', '--from_exp', required=True, help="original expid to copy from")
    parser.add_argument('-o', '--overwrite', action="store_true", default=False, help="overwrite original expid")
    args = parser.parse_args()

    # Creating experiment copy
    BasicConfig.read()
    from_experiment_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, args.from_exp)
    from_as_conf = AutosubmitConfig(args.from_exp)
    from_exp_parser = from_as_conf.get_parser(from_as_conf.experiment_file)
    hpc = from_exp_parser.get('experiment', 'HPCARCH')
    description = "Experiment migrated from {0}".format(args.from_exp)

    # Check overwrite
    if args.overwrite:
        if Autosubmit._user_yes_no_query("Do you want to overwrite configuration for {0} ?".format(args.from_exp)):
            print '{1}WARNING: You are going to overwrite {0} configurations !!{2}'.format(
                args.from_exp, WARNING, DEFAULT)
            if Autosubmit._user_yes_no_query("Are you sure you want to continue ?"):
                to_exp = args.from_exp
            else:
                return False
    else:
        to_exp = Autosubmit.expid(hpc, description, args.from_exp, False)

    # Creating proj
    from_project_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, args.from_exp, "git")
    to_project_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, to_exp, BasicConfig.LOCAL_PROJ_DIR)
    os.mkdir(to_project_path)

    # Cloning previous git into proj
    status = dict()
    subprojects = ["templates", "model", "ocean_diagnostics"]
    for subproj in subprojects:
        try:
            fnull = open(os.devnull, 'w')
            status[subproj] = subprocess.call(['git', 'clone', os.path.join(from_project_path, subproj),
                                              os.path.join(to_project_path, subproj)],
                                              stdout=fnull, stderr=subprocess.STDOUT)
        except OSError as e:
            print 'Cloning {0}: {1}Failed, {3}{2}'.format(subproj, ERROR, DEFAULT, e)
            exit(1)

        if status[subproj] == 0:
            print 'Cloning {0}: {1}OK{2}'.format(subproj, RESULT, DEFAULT)

    # Removing unnecessary template header/tailer variables
    bad_vars = ["%HEADER%", "%AS-HEADER-LOC%", "%AS-HEADER-REM%", "%AS-TAILER-LOC%", "%AS-TAILER-REM%"]
    from_templates_path = os.path.join(from_experiment_path, "git", "templates")
    templates_sets = [name for name in os.listdir(from_templates_path)
                      if os.path.isdir(os.path.join(from_templates_path, name)) and not name.startswith('.')]
    for template_name in templates_sets:
        templates = [name for name in os.listdir(os.path.join(from_templates_path, template_name))
                     if os.path.isfile(os.path.join(from_templates_path, template_name, name))
                     and not name.startswith('.')]
        for temp in templates:
            with open(os.path.join(from_experiment_path, "git", "templates", template_name, temp)) as oldfile, open(
                    os.path.join(to_project_path, "templates", template_name, temp), 'w') as newfile:
                for line in oldfile:
                    if not any(bad_var in line for bad_var in bad_vars):
                        newfile.write(line)
    print 'Adapting template variables to Autosubmit 3.x: {0}OK{1}'.format(RESULT, DEFAULT)

    # Variables to add
    text_proj = textwrap.dedent(
        """
        # Calendar used. LIST: standard, noleap
        CALENDAR = standard

        [project]
        # Select project type. STRING = git, svn, local, none
        # If PROJECT_TYPE is set to none, Autosubmit self-contained dummy templates will be used
        PROJECT_TYPE = local

        # If PROJECT_TYPE is not git, no need to change
        [git]
        # Repository URL  STRING = 'https://github.com/torvalds/linux.git'
        PROJECT_ORIGIN =
        # Select branch or tag, STRING, default = 'master', help = {'master' (default), 'develop', 'v3.1b', ...}
        PROJECT_BRANCH =
        # type = STRING, default = leave empty, help = if model branch is a TAG leave empty
        PROJECT_COMMIT =

        # If PROJECT_TYPE is not svn, no need to change
        [svn]
        # type = STRING, help = 'https://svn.ec-earth.org/ecearth3'
        PROJECT_URL =
        # Select revision number. NUMERIC = 1778
        PROJECT_REVISION =

        # If PROJECT_TYPE is not local, no need to change
        [local]
        # type = STRING, help = /foo/bar/ecearth
        PROJECT_PATH = /cfu/autosubmit/""" + args.from_exp + """/git

        # If PROJECT_TYPE is none, no need to change
        [project_files]
        # Where is PROJECT CONFIGURATION file location relative to project root path
        FILE_PROJECT_CONF = ../conf/proj_""" + to_exp + """.conf

        [rerun]
        # Is a rerun or not? [Default: Do set FALSE]. BOOLEAN = TRUE, FALSE
        RERUN = FALSE
        # If RERUN = TRUE then supply the list of chunks to rerun
        # LIST = "[ 19601101 [ fc0 [1 2 3 4] fc1 [1] ] 19651101 [ fc0 [16-30] ] ]"
        CHUNKLIST =
        """
    )
    # Commenting unnecessary configuration variables
    # Adding configuration variables
    to_as_conf = AutosubmitConfig(to_exp)
    bad_confs = ["HPCPROJ", "HPCUSER", "RERUN", "CHUNKLIST", "WALLCLOCK", "NUMPROC"]
    from_file = to_as_conf.experiment_file + ".old"
    shutil.copyfile(to_as_conf.experiment_file, from_file)
    with open(from_file) as oldfile, open(to_as_conf.experiment_file, 'w') as newfile:
        for line in oldfile:
            if not any(bad_conf in line for bad_conf in bad_confs):
                newfile.write(line)
            else:
                newfile.write("# migrate_exp: variable REMOVED! (Autosubmit 2.x -> Autosubmit 3.x)\n")
            if line.startswith("CHUNKINI"):
                newfile.write(text_proj)
                newfile = open(to_as_conf.project_file, 'w')
                newfile.write("[common]\n")
        print 'Adapting expdef conf variables to Autosubmit 3.x: {1}OK{2}'.format(
            to_as_conf.experiment_file, RESULT, DEFAULT)

    # Preparing jobs and queues conf
    new_conf_files = ['platforms.conf', 'jobs.conf']
    for filename in new_conf_files:
        if resource_exists('autosubmit', 'config/files/' + filename):
            index = filename.index('.')
            new_filename = filename[:index] + "_" + to_exp + filename[index:]

        if filename == 'platforms.conf' and BasicConfig.DEFAULT_PLATFORMS_CONF != '':
            content = file(os.path.join(BasicConfig.DEFAULT_PLATFORMS_CONF, filename)).read()
            file(BasicConfig.LOCAL_ROOT_DIR + "/" + to_exp + "/conf/" + new_filename, 'w').write(content)
            print 'New config file {0} created: {1}OK{2}'.format(filename, RESULT, DEFAULT)
        elif filename == 'jobs.conf' and BasicConfig.DEFAULT_JOBS_CONF != '':
            content = file(os.path.join(BasicConfig.DEFAULT_JOBS_CONF, filename)).read()
            file(BasicConfig.LOCAL_ROOT_DIR + "/" + to_exp + "/conf/" + new_filename, 'w').write(content)
            print 'New config file {0} created: {1}OK{2}'.format(filename, RESULT, DEFAULT)

if __name__ == "__main__":
    main()

